/*
 *  SoraMoyo.js
 */

// Objects
var presets;
var areaSelector;
var styleCollection;
var widgetPref;
var areaDictionary;
var frontDisplay;
var forecast;
var forecastDataStorage;
var display = "regular";
var lastLoaded = {time:null, area:null};
var refleshTimerID = null;
var refreshInterval = 60*60*1000 + Math.round((Math.random() - 0.5) * 60 * 1000);
var loadingMode;
var jmaURI = "http://www.jma.go.jp/";
var websiteURI = "http://www.cyanworks.net/dboardSoraMoyo.html";

// Window resize
var windowWidth = 314;
var windowHeight = 130;
var windowWidthRev = 314;
var windowHeightRev = 200;

// Animation objects
var switchButton;
var detailArea;

// Initialize widget
function init() {
    var wait = 0.5 * 1000 + Math.round(Math.random() * 0.5 * 1000) // Wait 0.5 - 1 sec

	// Animation objects
	switchButton = new Fader("switchbutton", false, "front");
	detailArea = new Fader("detail", false);

	// Objects
	styleCollection = new StyleCollection();
	widgetPref = new WidgetPreference();
	areaDictionary = new AreaDictionary();
	presets = new Presets();
	areaSelector = new AreaSelector();

	if (window.widget) {
		widget.onshow = onShow;
		widget.onhide = onHide;
		widget.onremove = onRemove;
	}

	forecast = new Forecast();
	forecastDataStorage = new Array();

	// Events
	document.addEventListener("mouseout", mouseOut, true);
	document.addEventListener("mouseover", mouseOver, true);
	document.addEventListener("keydown", onKeyDown, true);
	document.addEventListener("keyup", onKeyUp, true);

	frontDisplay = new FrontDisplay();

	// For test
//	jmaURI = "http://localhost/~cyan/TestSites/DOMTest/";
//	refreshInterval = 3* 60 * 1000;

	// weather data loading start
	lastLoaded.time = 0;
	lastLoaded.area = new ForecastArea();
	reload(wait);
}

function onShow() {
	Fader.interval = 13;
}

function onHide() {
	Fader.interval = 1000;
}

function onRemove() {
	widgetPref.removeMyPref();
}

// Waked up from the sleep mode
function systemDidWake() {
	var now = (new Date).getTime();
	var wait;

	LOG__("systemDidWake");

	if (0 < lastLoaded.time) {
		if (refreshInterval < (now - lastLoaded.time)) {
            wait = 10 * 1000 + Math.round(Math.random() * 2 * 1000); // Wait 10 - 12 sec
			LOG__("Current displaying data is too old. Reload wait: " + wait);
		} else {
            wait = 10 * 1000 + refreshInterval - (now - lastLoaded.time); // Recalculete the interval
			LOG__("Recalculete the interval. Reload wait: " + wait);
		}
        reload(wait);
	}
}

function Forecast() {
	LOG__("Object: Forecast");
	this.data = null;
	this.data3 = null;
	this.data7 = null;
	this.data7Length = 0;
	this.area = new ForecastArea(18, 0, 0); // Default area (Tokyo)

	// load the area from widgetPref
	var pref    = widgetPref.value("pref");
	var section = widgetPref.value("section");
	var city    = widgetPref.value("city");

	// If widgetPref does not have area data, use the default area.
	if ((pref != null) && (section != null) && (city != null))
		this.area.setAreaWithCode(pref, section, city);

	this.saveArea();
	this.setArea(this.area);
}

// Set area code and forecast data
Forecast.prototype.setArea = function(theArea) {
	LOG__("Forecast.setArea(theArea)");
	if (this.area.isEqualTo(theArea))
		return;

	this.area.setArea(theArea);
	this.saveArea();
}

// Update forecast.data3 with forecastDataStorage
Forecast.prototype.loadData3 = function() {
	LOG__("Forecast.loadData3()");
	var dataArray = this.data.data3;
	var sectionDic = areaDictionary.prefs[this.area.pref].array[this.area.section];
	var day;
	var data1;
	var newData1;

	if (!dataArray)
		return;

	// Reselect the city
	//   If not exist any temperture data, correct the city number
	switch (this.area.city) {
		case 0:
			break;
		case 1:
			if ((dataArray[this.area.section][0][7] == "n/a")
				&& (dataArray[this.area.section][0][8] == "n/a")
				&& (dataArray[this.area.section][1][7] == "n/a")
				&& (dataArray[this.area.section][1][8] == "n/a")
				&& (dataArray[this.area.section][2][7] == "n/a")
				&& (dataArray[this.area.section][2][8] == "n/a"))
				this.area.city = 0;
			break;
		case 1:
			if ((dataArray[this.area.section][0][9] == "n/a")
				&& (dataArray[this.area.section][0][10] == "n/a")
				&& (dataArray[this.area.section][1][9]  == "n/a")
				&& (dataArray[this.area.section][1][10] == "n/a")
				&& (dataArray[this.area.section][2][9]  == "n/a")
				&& (dataArray[this.area.section][2][10] == "n/a"))
				this.area.city = 0;
			break;
	}

	// Set 3days forecast data
	this.data3 = new Array();
	for (day = 0; day < 3; day++) {
		data1 = dataArray[this.area.section][day];
		newData1 = new ForecastData1day();

		newData1.date        = data1[0];
		newData1.weatherIcon = new WeatherIcon(data1[1]);
		newData1.weather     = data1[2];
		newData1.windAndWave = data1[3];
		newData1.rProv00     = data1[4][0];
		newData1.rProv06     = data1[4][1];
		newData1.rProv12     = data1[4][2];
		newData1.rProv18     = data1[4][3];
		switch (this.area.city) {
			case 0:
				newData1.tempL = data1[5];
				newData1.tempH = data1[6];
				break;
			case 1:
				newData1.tempL = data1[7];
				newData1.tempH = data1[8];
				break;
			case 2:
				newData1.tempL = data1[9];
				newData1.tempH = data1[10];
				break;
		}
		this.data3.push(newData1);
	}
}

// Update forecast.data7 with forecastDataStorage
Forecast.prototype.loadData7 = function() {
	LOG__("Forecast.loadData7()");
	var dataArray = this.data.data7;
	var sectionDic = areaDictionary.prefs[this.area.pref].array[this.area.section];
	var day;
	var data1;
	var newData1;
	var wSection;

	if (!dataArray)
		return;

	// Select section of 7days forecast
	this.data7Length = dataArray.length
	wSection = sectionDic["wCode" + this.data7Length];

	// Set 7days forecast data
	this.data7 = new Array();
	for (day = 0; day < 7; day++) {
		data1 = dataArray[wSection][day];
		newData1 = new ForecastData1day();
		newData1.date        = data1[0];
		newData1.weatherIcon = new WeatherIcon(data1[1]);
		newData1.rProv00     = data1[2];
		newData1.tempL       = data1[3];
		newData1.tempH       = data1[4];
		this.data7.push(newData1);
	}
}

Forecast.prototype.saveArea = function() {
	LOG__("Forecast.saveArea()");
	widgetPref.setValue(this.area.pref, "pref");
	widgetPref.setValue(this.area.section, "section");
	widgetPref.setValue(this.area.city, "city");
}

function reload(wait) {
	if (0 <= wait)
		LOG__("reload(" + wait/1000 + ")");
	else
		LOG__("reload()");

	if (refleshTimerID != null)
		clearTimeout(refleshTimerID);
	loadingMode = "3days";

	if (wait)
		refleshTimerID = setTimeout("loadWeather()", wait);
	else
		loadWeather();
}

function superReload() {
	var key;

	// clear cache
	forecastDataStorage["pref" + forecast.area.pref] = null;
	reload();
}

// Access to website
function loadWeather() {
	var i;
	var ifModSince = "";

	// Set forecast data
	forecast.data = forecastDataStorage["pref" + forecast.area.pref];
	if (!forecast.data) {
		forecast.data = new ForecastData();
		forecastDataStorage["pref" + forecast.area.pref] = forecast.data;
	}

	// Try to load the cache
	switch (loadingMode) {
		case "3days":
			if (forecast.data.lastModTime3)
				ifModSince = forecast.data.lastModTime3;
			break;
		case "7days":
			if (forecast.data.lastModTime7)
				ifModSince = forecast.data.lastModTime7;
			break;
	}

	LOG__("loadWeather " + loadingMode + " PREF:"
		+ forecast.area.pref + " (code " + forecast.area.prefCode
		+ ") SECTION:" + forecast.area.section
		+ " ifModSince:" + ifModSince);
	frontDisplay.setStatus("Loading.");

	if (window.widget && SoraMoyoPlugin)
		SoraMoyoPlugin.loadWeather(forecast.area.prefCode, loadingMode, jmaURI, ifModSince);
	else
		alert("Widget plugin not loaded.");
}

function stopLoading() {
	if (window.widget && SoraMoyoPlugin)
		SoraMoyoPlugin.stopLoading();
	else
		alert("Widget plugin not loaded.");
}

function downloadFinished(weatherData) {
	frontDisplay.setStatus("Finished.");
	storeWeather("new", weatherData);
}

function downloadNotModified() {
	frontDisplay.setStatus("Not modified.");
	storeWeather("cache", null);
}

function downloadError(error) {
	frontDisplay.setStatus("Error: " + error, true);

	// Restore last loaded area
	forecast.area.setArea(lastLoaded.area);

	// Next reflesh
	// lastLoaded.time = (new Date).getTime();
	reload(10*60*1000); // Wait 10 minutes and retry
}

// Store weather data
function storeWeather(mode, weatherData) {
	LOG__("storeWeather MODE: " + loadingMode + " (" + mode + ")");

	switch (loadingMode) {
		case "3days":
			if (mode == "new")
				forecast.data.set3days(weatherData);
			loadingMode = "7days";
			loadWeather();
			break;
		case "7days":
			if (mode == "new")
				forecast.data.set7days(weatherData);
			showWeather();

			// Next reflesh
			lastLoaded.time = (new Date).getTime();
			lastLoaded.area.setArea(forecast.area);
			reload(refreshInterval);
			LOG__("lastLoaded.time: " + (lastLoaded.time/1000)
				+ " next:" + ((lastLoaded.time + refreshInterval)/1000));
			break;
	}
}

// Show weather data
function showWeather() {
	var weatherArray = new Array();
	var i, j, k;
	var child, children;
	var icon;
	var weatherIconSplitter, weatherIconMini1, weatherIconMini2;
	var date, dateAlt;
	var now;
	var time = new Date();
	var lastModTime = new Date();
	var buffer;
	var day;
	var dayArray = ["SUN", "MON", "TUE", "WED", "THU", "FRI", "SAT"];
	var location = document.getElementById("location");
	var data1day;

	if (forecast.data == null) return;

	forecast.loadData3();
	forecast.loadData7();

	LOG__("forecast.data3.length:" + forecast.data3.length);
	LOG__("forecast.data7.length:" + forecast.data7.length);

	// Display location
	frontDisplay.setLocation(forecast.area, forecast.data7Length);

	// Last Modified
	buffer = "";
	if (forecast.data.lastModTime3 && forecast.data.lastModTime7) {
		lastModTime.setTime(Date.parse(forecast.data.lastModTime3));
		buffer += ("3 Days: " + lastModTime.toString());
		lastModTime.setTime(Date.parse(forecast.data.lastModTime7));
		buffer += ("\n7 Days: " + lastModTime.toString());
	}
	switchButton.target.setAttribute("title", buffer);

	// Display 3days forecast
	for (i = 0; i < 3; i++) {
		switch(i) {
			case 0:
				children = document.getElementById("today").childNodes;
				break;
			case 1:
				children = document.getElementById("tomorrow").childNodes;
				break;
			case 2:
				children = document.getElementById("datomorrow").childNodes;
				break;
		}
		data1day = forecast.data3[i];

		// Check all elements
		for (j = 0; j < children.length; j++) {
			child = children[j]

			// Display date
			if (child.className == "date") {
				while (child.hasChildNodes())
					child.removeChild(child.lastChild);

				now = (new Date).getTime();
				for (k = -10; k < 14; k++) {
					time.setTime(now + k*24*60*60*1000);
					date = time.getDate();
					if (data1day.date == date) {
						child.innerText = date;
						dateAlt = document.createElement("span");
						dateAlt.innerText = getLocalizedString(dayArray[time.getDay()]);
						dateAlt.setAttribute("class", "dateAlt");
						child.appendChild(dateAlt);
					}
				}
			}

			var img;
			// Display weather icon
			if (child.className == "weather") {
				while (child.hasChildNodes())
					child.removeChild(child.lastChild);

				if (data1day.weatherIcon.icon != "n/a") {
					if (data1day.weatherIcon.icon) {
						img = document.createElement("img");
						img.src = frontDisplay.iconsPath + data1day.weatherIcon.icon + ".png";
						img.setAttribute("class", "weathericon");
						child.appendChild(img);
					} else {
						img = document.createElement("img");
						img.src = frontDisplay.iconsPath + data1day.weatherIcon.icon1 + ".png";
						img.setAttribute("class", "weathericonmini1");
						child.appendChild(img);

						img = document.createElement("img");
						img.src = frontDisplay.iconsPath + data1day.weatherIcon.icon2 + ".png";
						img.setAttribute("class", "weathericonmini2");
						child.appendChild(img);

						img = document.createElement("img");
						img.src = frontDisplay.iconsPath + data1day.weatherIcon.splitter + ".png";
						img.setAttribute("class", "weathericonsplitter");
						child.appendChild(img);
					}
				} else { // Weather data not available yet
					if (i == 2) {
						img = document.createElement("img");
						img.src = frontDisplay.iconsPath + "weatherNAY" + ".png";
						img.setAttribute("class", "weathernay");
						child.appendChild(img);
					}
				}
			}

			// Display min/max temperture
			if (child.className == "tempL") {
				child.innerHTML = "";
				if (data1day.tempL != "n/a")
					child.innerHTML
						= data1day.tempL.replace(/[^0-9\-][0-9\-]+[^0-9\-]/, "") + "&#x00b0;";
			}
			if (child.className == "tempH") {
				child.innerHTML = "";
				if (data1day.tempH != "n/a")
					child.innerHTML
						= data1day.tempH.replace(/[^0-9\-][0-9\-]+[^0-9\-]/, "") + "&#x00b0;";
			}

			// Display precipitation probability
			if (child.className == "rProv00") {
				if (data1day.rProv00 != "n/a")
					child.innerHTML = data1day.rProv00;
				else
					child.innerHTML = "";
			}

			if (child.className == "rProv06") {
				if (data1day.rProv06 != "n/a")
					child.innerHTML = data1day.rProv06;
				else
					child.innerHTML = "";
			}

			if (child.className == "rProv12") {
				if (data1day.rProv12 != "n/a")
					child.innerHTML = data1day.rProv12;
				else
					child.innerHTML = "";
			}

			if (child.className == "rProv18") {
				if (data1day.rProv18 != "n/a")
					child.innerHTML = data1day.rProv18;
				else
					child.innerHTML = "";
			}

			if (child.className == "rProvRight") {
				if (data1day.rProv18 != "n/a")
					child.innerHTML = "%";
				else
					child.innerHTML = "";
			}
		}
	}

	// Display 7days forecast
	for (i = 0; i < 7; i++) {
		switch(i) {
			case 0:
				children = document.getElementById("day0").childNodes;
				break;
			case 1:
				children = document.getElementById("day1").childNodes;
				break;
			case 2:
				children = document.getElementById("day2").childNodes;
				break;
			case 3:
				children = document.getElementById("day3").childNodes;
				break;
			case 4:
				children = document.getElementById("day4").childNodes;
				break;
			case 5:
				children = document.getElementById("day5").childNodes;
				break;
			case 6:
				children = document.getElementById("day6").childNodes;
				break;
		}
		data1day = forecast.data7[i];

		// Check all elements
		for (j = 0; j < children.length; j++) {
			child = children[j]

			// Display date
			if (child.className == "date") {
				while (child.hasChildNodes())
					child.removeChild(child.lastChild);

				now = (new Date).getTime();
				for (k = -7; k < 14; k++) {
					time.setTime(now + k*24*60*60*1000);
					date = time.getDate();
					if (data1day.date == date) {
						child.innerText = date;
						dateAlt = document.createElement("span");
						dateAlt.innerText = getLocalizedString(dayArray[time.getDay()]);
						dateAlt.setAttribute("class", "dateAlt");
						child.appendChild(dateAlt);
					}
				}
			}

			if (child.className == "weather") {
				while (child.hasChildNodes())
					child.removeChild(child.lastChild);

				if (data1day.weatherIcon.icon != "n/a") {
					if (data1day.weatherIcon.icon) {
						img = document.createElement("img");
						img.src = frontDisplay.iconsPath + data1day.weatherIcon.icon + ".png";
						img.setAttribute("class", "weathericon");
						child.appendChild(img);
					} else {
						img = document.createElement("img");
						img.src = frontDisplay.iconsPath + data1day.weatherIcon.icon1 + ".png";
						img.setAttribute("class", "weathericonmini1");
						child.appendChild(img);

						img = document.createElement("img");
						img.src = frontDisplay.iconsPath + data1day.weatherIcon.icon2 + ".png";
						img.setAttribute("class", "weathericonmini2");
						child.appendChild(img);

						img = document.createElement("img");
						img.src = frontDisplay.iconsPath + data1day.weatherIcon.splitter + ".png";
						img.setAttribute("class", "weathericonsplitter");
						child.appendChild(img);
					}
				}
			}

			// Display min/max temperture
			if (child.className == "rProv") {
				if ((data1day.rProv00 != "n/a") && (data1day.rProv00.length <= 3))
					child.innerHTML = data1day.rProv00 + "%";
				else
					child.innerHTML = "";
			}

			// Display min/max temperture
			if (child.className == "tempL") {
				child.innerHTML = "";
				if (data1day.tempL != "n/a")
					child.innerHTML
						= data1day.tempL.replace(/[^0-9\-][0-9\-]+[^0-9\-]/, "") + "&#x00b0;";
			}
			if (child.className == "tempH") {
				child.innerHTML = "";
				if (data1day.tempH != "n/a")
					child.innerHTML
						= data1day.tempH.replace(/[^0-9\-][0-9\-]+[^0-9\-]/, "") + "&#x00b0;";
			}
		}
	}
}

function ForecastData() {
	LOG__("Object: ForecastData");
	this.lastModTime3 = null;
	this.lastModTime7 = null;

	// today, tomorrow, the day after tomorrow
	this.data3 = new Array();
	this.data7 = new Array();
}

ForecastData.prototype.set3days = function(dataArray) {
	LOG__("ForecastData.set3days[" + dataArray.length + "]");

	this.lastModTime3 = dataArray[0]; // Last-Modified
	this.data3 = dataArray.slice(1);
}

ForecastData.prototype.set7days = function(dataArray) {
	LOG__("ForecastData.set7days[" + dataArray.length + "]");

	this.lastModTime7 = dataArray[0]; // Last-Modified
	this.data7 = dataArray.slice(1);
}

function ForecastData1day() {
	this.date        = null;
	this.weatherIcon = null;
	this.weather     = null;
	this.windAndWave = null;
	this.rProv00     = null;
	this.rProv06     = null;
	this.rProv12     = null;
	this.rProv18     = null;
	this.tempL       = null;
	this.tempH       = null;
}

function WeatherIcon(iconString) {
	this.string   = iconString;
	var k;
	var icon = null;
	var splitter = null;
	var icon1 = null;
	var icon2 = null;

	if (iconString != "n/a") {
		if (0 < iconString.indexOf("|")) {
			i = iconString.indexOf("|");
			icon1 = iconString.substring(0, i);
			icon2 = iconString.substring(i + 1);
			splitter = "occas";
		} else if (0 < iconString.indexOf("/")) {
			i = iconString.indexOf("/");
			icon1 = iconString.substring(0, i);
			icon2 = iconString.substring(i + 1);
			splitter = "after";
		} else {
			icon = iconString;
		}
	} else {
		icon = "n/a";
	}

	this.icon     = icon;
	this.splitter = splitter;
	this.icon1    = icon1;
	this.icon2    = icon2;
}

// ================================================
// FrontDisplay
// ================================================
function FrontDisplay() {
	LOG__("Object: FrontDisplay");
	var divs, option;
	var i, len;

	this.status = document.getElementById("status");
	this.iconsPath = "Images/";

	// Prepare styles
	this.styles = new Array;
	for(i = 0, len = styleCollection.items.length; i < len; i++) {
		option = document.createElement("option");
		option.innerText = styleCollection.items[i].title;
		option.value = i;
		document.getElementById("styleSelect").appendChild(option);
	}

	// Status line
	this.statusLine = new Fader("status", false);
	this.statusLine.outDuration = 4000;

	// Set properties
	this.setStyleNum(widgetPref.value("styleNum"));
	this.setMode(widgetPref.value("dispMode"));
}

FrontDisplay.prototype.setMode = function(value) {
	var regular = document.getElementById("regular");
	var week = document.getElementById("week");

	LOG__("FrontDisplay.setDisplay(" + value + ")");
	var newValue = value;

	if ((value == null)
		|| !((value == "regular") || (value == "week") || (value == "++")))
		newValue = "regular";

	if (value == "++") {
		if (this.mode != null)
			newValue = (this.mode== "regular")? "week" : "regular";
		else
			newValue = "regular";
	}

	switch(newValue) {
		case "regular":
			regular.style.display  = "block";
			week.style.display= "none";
			switchButton.setImage("Images/switch3.png");
			document.getElementById("section").style.display = "inline";
			document.getElementById("wSection").style.display = "none";
			document.getElementById("splitter").setAttribute("class", "splitter3");
			break;
		case "week":
			regular.style.display = "none";
			week.style.display = "block";
			switchButton.setImage("Images/switch7.png");
			document.getElementById("section").style.display = "none";
			document.getElementById("wSection").style.display = "inline";
			document.getElementById("splitter").setAttribute("class", "splitter7");
			break;
	}

	widgetPref.setValue(newValue, "dispMode");
	this.mode = newValue;
}

FrontDisplay.prototype.setStyleNum = function(value) {
	LOG__("FrontDisplay.setStyleNum(" + value + ")");
	var len = styleCollection.items.length;
	var linkCSS = document.getElementById("linkCSS");
	var newValue = value;
	var node, newStyle, i, icons;

	if ((value == null) || value != (Math.floor(value))
			|| (value < 0) || (len <= value))
		newValue = 0;

	if (value == "++") {
		if (this.styleNum != null)
			newValue = (this.styleNum < len - 1)? this.styleNum + 1 : 0;
		else
			newValue = 0;
	}

	if (value == "--") {
		if (this.styleNum != null)
			newValue = (0 < this.styleNum)? this.styleNum - 1 : len - 1;
		else
			newValue = 0;
	}

	if (this.styleNum == newValue) return;

	newStyle = styleCollection.items[newValue]
	linkCSS.setAttribute("href", newStyle.href);
	this.iconsPath = "WeatherIcons/" + newStyle.icons + "/";

	windowWidth = newStyle.width;
	windowHeight = newStyle.height;
	if (window.widget)
		window.resizeTo(windowWidth, windowHeight);

	this.styleNum = newValue;
	this.setStatus("Style: " + newStyle.title, false);
	simulateMouseOut();
	document.getElementById("styleSelect").selectedIndex = newValue;
	widgetPref.setValue(newValue, "styleNum");
	showWeather();
}

/*
FrontDisplay.prototype.fitWindowSize = function() {
	LOG__("FrontDisplay.fitWindowSize()");

	var front      = document.getElementById("front");
	var frontStyle = document.defaultView.getComputedStyle(front,"");
	var width  = parseInt(frontStyle.getPropertyValue("width"));
	var height = parseInt(frontStyle.getPropertyValue("height"));

	windowWidth = width;
	windowHeight = height;

	if (window.widget)
		window.resizeTo(width, height);
}
*/

FrontDisplay.prototype.setStatus = function(str, important) {
	LOG__("FrontDisplay.setStatus(" + str + ")");

	this.status.innerText = str;
	this.statusLine.setOpacity(1.0);

	if (!important)
		this.statusLine.fadeOut();
}

// Display Location
FrontDisplay.prototype.setLocation = function(theArea, data7Length) {
	LOG__("FrontDisplay.setLocation(theArea)");

	var prefLine = document.getElementById("pref");
	var sectionLine = document.getElementById("section");
	var wSectionLine = document.getElementById("wSection");

	// Prepare styles
	sectionLine.style.display = "inline";
	wSectionLine.style.display = "inline";

	// pref, section, city
	prefLine.innerText = getLocalizedString(theArea.stringPref());
	sectionLine.innerText =
		getLocalizedString(theArea.stringSection()) + " - " +
		getLocalizedString(theArea.stringCity());

	// section (7 days)
	wSection.innerText = getLocalizedString(theArea.stringWSection(data7Length));

	// Restore styles
	switch(this.mode) {
		case "regular":
			sectionLine.style.display = "inline";
			wSectionLine.style.display = "none";
			break;
		case "week":
			sectionLine.style.display = "none";
			wSectionLine.style.display = "inline";
			break;
	}
}

//Mouse Event
function mouseOut(e) {
	var node;

	// Hide detail panel
	if (e.target.className) {
		if ((e.target.className.match(/weathericon/))
			|| (e.target.className.match(/weathernay/)) )
		{
			document.getElementById("detail").style.zIndex = 0;
			detailArea.fadeOut();
		}
	}
}

function mouseOver(e) {
	// Show detail panel
	if (e.target.className) {
		if ((e.target.className.match(/weathericon/))
			|| (e.target.className.match(/weathernay/)) )
		{
			showDetail(e.target.parentNode.parentNode.id);
		}
	}
}

function onKeyDown(event) {
	LOG2__("onKeyDown:" + event.charCode);
	var handled = true;

	if (event.metaKey) return;

	if (display != "reverse") {
		switch (event.charCode) {
			case 100:	// d
			case 68:	// D
			case 114:	// r
			case 82:	// R
			case 115:	// s
			case 83:	// S
			case 46:
				break;
			default:
				handled = false;
				break;
		}
	}

	switch (event.charCode) {
		case 49:	// 1
		case 50:	// 2
		case 51:	// 3
		case 52:	// 4
		case 119:	// w
		case 87:	// W
			handled = true;
	}

	if (handled) {
		event.stopPropagation();
		event.preventDefault();
	}
}

function onKeyUp(event) {
	LOG2__("onKeyUp:" + event.charCode);
	var handled = true;

	if (event.metaKey) return;

	if (display == "regular") {
		switch (event.charCode) {
			case 100:	// d
			case 68:	// D
				frontDisplay.setMode("++");
				break;
			case 114:	// r
			case 82:	// R
				if (!event.shiftKey)
					reload();
				else
					superReload();
				break;
			case 115:	// s
			case 83:	// S
				if (!event.shiftKey)
					frontDisplay.setStyleNum("++");
				else
					frontDisplay.setStyleNum("--");
				break;
			case 46:
				stopLoading();
				break;
			case 49:	// 1
				forecast.setArea(presets.area[0]);
				reload();
				break;
			case 50:	// 2
				forecast.setArea(presets.area[1]);
				reload();
				break;
			case 51:	// 3
				forecast.setArea(presets.area[2]);
				reload();
				break;
			case 52:	// 4
				forecast.setArea(presets.area[3]);
				reload();
				break;
			case 119:	// w
			case 87:	// W
				openWebsite(websiteURI);
				break;
			default:
				handled = false;
				break;
		}
	}

	if (display == "reverse") {
		switch (event.charCode) {
			case 49:	// 1
				areaSelector.setArea(presets.area[0]);
				break;
			case 50:	// 2
				areaSelector.setArea(presets.area[1]);
				break;
			case 51:	// 3
				areaSelector.setArea(presets.area[2]);
				break;
			case 52:	// 4
				areaSelector.setArea(presets.area[3]);
				break;
			default:
				handled = false;
				break;
		}
	}

	if (handled) {
		event.stopPropagation();
		event.preventDefault();
	}
}

// Show detail panel
function showDetail(date) {
	var weatherArray = new Array();
	var detail = document.getElementById("detail");
	var buffer = "";
	var weather;
	var windAndWave;
	var re;

	switch(date) {
		case "today":
			weather		= forecast.data3[0].weather;
			windAndWave	= forecast.data3[0].windAndWave;
			break;
		case "tomorrow":
			weather		= forecast.data3[1].weather;
			windAndWave	= forecast.data3[1].windAndWave;
			break;
		case "datomorrow":
			if (forecast.data3[2].weatherIcon.icon != "n/a") {
				weather		= forecast.data3[2].weather;
				windAndWave	= forecast.data3[2].windAndWave;
			} else {
				weather = getLocalizedString("Forecast Data not available yet.")
			}
			break;
	}

	if (weather && (weather != "") && (weather != "n/a"))
		buffer = weather;
	if (windAndWave && (windAndWave != "") && (windAndWave != "n/a"))
		buffer += ("&nbsp;&nbsp;&nbsp;&nbsp;" + windAndWave);

	buffer = buffer.replace(RegExp(String.fromCharCode(0xff10),"gm"), "0");
	buffer = buffer.replace(RegExp(String.fromCharCode(0xff11),"gm"), "1");
	buffer = buffer.replace(RegExp(String.fromCharCode(0xff12),"gm"), "2");
	buffer = buffer.replace(RegExp(String.fromCharCode(0xff13),"gm"), "3");
	buffer = buffer.replace(RegExp(String.fromCharCode(0xff14),"gm"), "4");
	buffer = buffer.replace(RegExp(String.fromCharCode(0xff15),"gm"), "5");
	buffer = buffer.replace(RegExp(String.fromCharCode(0xff16),"gm"), "6");
	buffer = buffer.replace(RegExp(String.fromCharCode(0xff17),"gm"), "7");
	buffer = buffer.replace(RegExp(String.fromCharCode(0xff18),"gm"), "8");
	buffer = buffer.replace(RegExp(String.fromCharCode(0xff19),"gm"), "9");
	buffer = buffer.replace(RegExp(String.fromCharCode(0xff0d),"gm"), "-");
	buffer = buffer.replace(RegExp(String.fromCharCode(0xff0e),"gm"), ".");
	buffer = buffer.replace(RegExp(String.fromCharCode(0xff05),"gm"), "%");

	buffer = buffer.replace(RegExp(String.fromCharCode(0x30E1,0x30fc,0x30c8,0x30eb),"gm"),
				" " + String.fromCharCode(0x334d)); // meter
	
	if (buffer != "") {
		detail.style.zIndex = 3;
		detail.innerHTML = buffer;
		detailArea.fadeIn();
	}
}

function switchMode() {
	frontDisplay.setMode('++');
}

// Open website
function openWebsite(uri) {
	if (window.widget)
		widget.openURL(uri);
	else
		window.location.href = uri;
}

function openWeatherPage() {
	var uri;

	if (frontDisplay.mode == "regular")
		uri = jmaURI + "jp/yoho/" + forecast.area.prefCode + ".html";
	if (frontDisplay.mode == "week")
		uri = jmaURI + "jp/week/" + forecast.area.prefCode + ".html";

	openWebsite(uri);
}

// Dispatch the mouseout event (hide buttons when widget is flipped)
function simulateMouseOut() {
	var front = document.getElementById("front");
	var infoButton = document.getElementById("infobutton");
	var event = document.createEvent("MouseEvents");
	event.initEvent("mouseout", true, true);

	infoButton.dispatchEvent(event);
	front.dispatchEvent(event);
}

// Reverse sides and preferences
function showBack(event) {
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	areaSelector.setArea(forecast.area);

	if (window.widget) {
		if (window.outerWidth < windowWidthRev)
			window.resizeBy(windowWidthRev - window.outerWidth, 0);

		if (window.outerHeight < windowHeightRev)
			window.resizeBy(0, windowHeightRev - window.outerHeight);
	}
	setTimeout ("simulateMouseOut();", 0);
	if (window.widget)
		widget.prepareForTransition("ToBack");

	back.style.display = "block";
	front.style.display = "none";
	display = "reverse";

	if (window.widget)
		setTimeout ("widget.performTransition();", 0);
}

function showFront(event) {
	var front = document.getElementById("front");
	var back = document.getElementById("back");

	if (window.widget) {
		if (window.outerWidth < windowWidth)
			window.resizeBy(windowWidth - window.outerWidth, 0);

		if (window.outerHeight < windowHeight)
			window.resizeBy(0, windowHeight - window.outerHeight);
	}

	if (window.widget)
		widget.prepareForTransition("ToFront");

	front.style.display = "block";
	back.style.display = "none";
	display = "regular";

	// Save preferences
	frontDisplay.setStyleNum(document.getElementById("styleSelect").selectedIndex);

	if (! forecast.area.isEqualTo(areaSelector.area)) {
		LOG__("Area is changed.");
		forecast.setArea(areaSelector.area);
		reload();
	}

	if (window.widget)
		setTimeout ("widget.performTransition();", 0);
}

// ================================================

// Debugging
function LOG__(arg) {
//	alert(arg);
}

function LOG2__(arg) {
//	alert(arg);
}

function LOG3__(arg) {
//	alert(arg);
}

// ================================================

/*
	forecast
		.area
			.pref
			.section
			.city
		.data
		.data3
		.data7

	forecastDataStorage
		.pref1
			.lastModTime3
			.lastModTime7
			.data3
			.data7
		.pref2
		.pref3
		...

	[Forecast loading process]
		Set forecast.area
		last-loaded-time
	
		Read last-modefied-time of 3days forecast from forecastDataStorage
		Download 3days forecast from JMA server
		Store data on forecastDataStorage (If updated)

		Read last-modefied-time of 7days forecast from forecastDataStorage
		Download 7days forecast from JMA server
		Store data on forecastDataStorage (If updated)

		Update forecast.data3 and .data7 with forecastDataStorage
		Reflesh display with forecast.data3 and .data7
*/

